!ver.03.01.19
!
! This is the FORTRAN 90 program that
! calculate solar direct and scattered flux 
! received by a solar panel on Martian surface
! in the visible solar spectrum
! 
! Laboratoire de Meteorologie Dynamique (LMD/CNRS) 2006-2019
! Modified in 2012 to run on MAC; no changes of results for LINUX
!

   !!!--- PART 1 ---!!!
!Comment out the following line to have random number generation by clocktime
#define _SEEDING_OFF_    
! General constants 
module constants
  real(8), public :: Pi, Pi2
  data Pi /3.141592653589793238462643383279502884197/
  data Pi2 /6.283185307179586476925286766559005768394/
  integer, public :: seed ! shared seed for random generator
  
  real(8), public :: arg_min
  data arg_min /1.0e-30/
end module constants

module test
 real(8), dimension(:,:), allocatable :: test_fin  ! flux received
 real(8), dimension(:,:), allocatable :: test_fout ! flux reflected

 real(8), dimension(:,:), allocatable :: test_fain  ! Spectrally integrated angular flux received
 real(8), dimension(:,:), allocatable :: test_faout ! Spectrally integrated angular flux reflected
end module test

module camera
 integer :: cam_ndir                              ! number of camera directions
 real(8) :: cam_opening                           ! Angular half-opening of camera
 real(8) :: cam_copening                          ! cosine of half-opening angle of camera
 real(8), dimension(:), allocatable :: cam_dira   ! camera Azimuth angle {cam_ndir}
 real(8), dimension(:), allocatable :: cam_dirz   ! camera Zenith angle {cam_ndir}
 real(8), dimension(:), allocatable :: cam_a      ! azimuth angle of camera axis {cam_ndir}
 real(8), dimension(:,:), allocatable :: cam_ft   ! flux total     {num of wavelength, cam_ndir}
 real(8), dimension(:,:), allocatable :: cam_fr   ! flux reflected {num of wavelength, cam_ndir}
 real(8), dimension(:,:), allocatable :: cam_fs   ! flux scattered {num of wavelength, cam_ndir}
 real(8), dimension(:,:), allocatable :: cam_fd   ! flux direct    {num of wavelength, cam_ndir}
 real(8), dimension(:,:), allocatable :: cam_vn
 real(8), dimension(:), allocatable   :: cam_sz
 
 character(30) :: cam_spectral                    ! filename for output 
end module camera

module boundary    
 real(8), dimension(:), allocatable :: top_fin      ! total flux entering on the top of the atmosphere
 real(8), dimension(:), allocatable :: top_fout     ! total flux outgoing on the top of the atmosphere
 
 character(30) :: top_spectral                    ! filename for the output on the top of the atmosphere
 
 character(30) :: sa_spectral                     ! filename for the spectral output on the SA
 character(30) :: sa_angular                      ! filename for the angular output on the SA
end module boundary    

! ***************************************************************

   !!!--- PART 2 ---!!!

! The main prgram

! Input (read from file "input"):
!   Number of photons launched per spectral band
! Geometry setup
!  Ls: Solar longitude (deg) (can be replaced by Earth time)
!      default 0.0
!  Lat: Latitude of solar panel on the planet (deg)
!       default 0.0
!  LTST: Local True Solar Time (hour, e.g. 12.5 for 12h30m)
!        default 12.0
! Atmosphere setup 
!  Albedo: Spectrally integrated surface Lambertian reflection albedo
!        default 0.2
!  SType: Surface spectral profile type 
!         1. Ockert-Bell et al., 1996 (default)
!         2. Bellucci et al., 2006 bright Olympus profile
!         3. Bellucci et al., 2006 dark Oxia Palus profile
!  TauD: Dust optical depth at reference wavelength (0.67 micron)
!        default 0.5
!  TauC: Cloud optical depth at reference wavelength (0.67 micron)
!        default 0.0
!  CloudH: Cloud height (given by dust optical thickness below cloud)
!        default 0.7*TauD
!  NSP: Number of solar panel direction
!  SPZ: Zenith angle of normal vector to solar panel 
!         compared to local zenith (deg) = panel slope...
!         SPZ = 0 -> normal vector is vertical
!                    (solar panel horizontal)
!       default 0.0
!  SPA: Azimuth angle of normal vector to solar panel
!          compared to local North (deg)
!          SPA = 0 -> Northward
!          SPA = 90 -> Eastward
!          SPA = 180 -> Southward
!          SPA = 270  -> Westward
!       default 0.0

! Output
! 1. (writen to file "output-spectral"):
!  Spectrum of flux received by the solar panel 
!
! 2. (writen to file "output-angular"):
!  ratio of flux received by solar panel within given incident
!  angle band, relative to normal vector of solar panel

! Author: Tran The Trung, Francois Forget, Ehouarn Millour,
!         Vladimir Zakharov
!         Laboratoire de Meteorologie Dynamique
!         France 2006-2019

program main
use constants
use test
use camera
use boundary
implicit none

!-- DECLARATION OF VARIABLES -- ! 

! inputs
!integer :: Np ! Number of photons launched per spectral band
integer :: Np_av ! average Number of photons launched in a spectral band
integer(8) :: Np_tot ! Total Number of photons launched in all spectral bands
real(8) :: Ls ! Solar longitude (degree)
real(8) :: Lat ! Latitude of solar panel on the planet (degree)
real(8) :: LTST ! Local True Solar Time (1 sol = 24h; 0 = midnight)
real(8) :: TauD ! Optical depth of dust at referencing wavelength ()
real(8) :: TauC ! Optical depth of cloud at referencing wavelength ()
real(8) :: CloudH ! Height of cloud layer
                            ! in term of optical thickness from surface
real(8) :: Albedo ! Spectrally integrated surface albedo
integer :: SType ! Surface spectral profile type
integer, parameter :: NSPMax=1000 ! Over-dimensioning to avoid allocation
integer :: NSP ! Number of solar panel direction
real(8), dimension(NSPMax) :: SPZ ! Zenith angle of normal vector to solar panel
!             compared to local zenith (degree)
!             SPZ = 0 -> normal vector is vertical
!                        (solar panel horizontal)
real(8), dimension(NSPMax) :: SPA ! Azimuth angle of normal vector to solar panel
!             compared to local North (degree)
!             SPA = 0 -> North
!             SPA = Pi/2 -> East
!             SPA = Pi -> South
!             SPA = 3Pi/2 -> West

! data
! solar spectrum
integer :: Nwl ! number of wavelength
integer, parameter :: NwlMax=10000 ! Over-dimensioning to avoid allocation
real(8), dimension(NwlMax) :: wl ! list of wavelength
real(8), dimension(NwlMax) :: ss ! solar spectrum power   

! outputs
real(8), dimension(:,:), allocatable :: ft ! total flux received by panel
real(8), dimension(:,:), allocatable :: fs ! scattered flux received by panel
real(8), dimension(:,:), allocatable :: fd ! direct flux received by panel
real(8), dimension(:,:), allocatable :: fr ! received surface reflected flux 
integer, parameter :: nmu = 90 ! Number of bin in the hemisphere above
real(8), dimension(:,:), allocatable :: fat, fas, far, fad ! Spectrally integrated angular flux

! private
integer :: i ! loop counting
real(8) :: dw2

real(8) :: x,ss_sum,ss_av,ssr_max
integer(8) :: Np_sum
integer, dimension(NwlMax) :: Np ! Number of photons launched per spectral band
character(30) :: input
character(30) :: ssname          ! filename with solar spectrum
integer*4     :: status

call GETARG(1, input)
status = IARGC()

if(status.eq.0) then
 print*,'No input file'
 STOP
else
 print*,'input file: ',input
endif

!-- FLOW OF COMPUTATION --!

! read inputs
call readinput(input,ssname,Np_av, Ls, Lat, LTST, NSP, NSPMax, SPZ, SPA, Albedo,&
                     SType, TauD, TauC, CloudH)

! read solar spectrum data
call readss(ssname, Nwl, NwlMax, wl, ss)

Np_tot = real(Np_av)*Nwl
ssr_max = 10.0 ! max ratio ss(i)/ss_av

ss_sum = 0.0
do i=1,Nwl
 ss_sum = ss_sum + ss(i)   
enddo
ss_av = ss_sum/real(Nwl)

Np_sum = 0
do i=1,Nwl
 x = ss(i)/ss_av
 if(x.gt.ssr_max) x = ssr_max
 if(x.lt.1.0/ssr_max) x = 1.0/ssr_max
 Np(i) = Np_tot/real(Nwl)*x
 if(Np(i).lt.1) Np(i)=1
 Np_sum = Np_sum + Np(i)
enddo

do i=1,Nwl
 Np(i) = Np(i)*(Np_tot/real(Np_sum))
 if(Np(i).lt.1) Np(i)=1
enddo

! do computations
if (allocated(ft)) deallocate(ft)
allocate(ft(Nwl,NSP))
if (allocated(fs)) deallocate(fs)
allocate(fs(Nwl,NSP))
if (allocated(fd)) deallocate(fd)
allocate(fd(Nwl,NSP))
if (allocated(fr)) deallocate(fr)
allocate(fr(Nwl,NSP))

if (allocated(fat)) deallocate(fat)
allocate(fat(nmu,NSP))
if (allocated(fas)) deallocate(fas)
allocate(fas(nmu,NSP))
if (allocated(fad)) deallocate(fad)
allocate(fad(nmu,NSP))
if (allocated(far)) deallocate(far)
allocate(far(nmu,NSP))

if (allocated(top_fin)) deallocate(top_fin)
allocate(top_fin(Nwl))
if (allocated(top_fout)) deallocate(top_fout)
allocate(top_fout(Nwl))
!
! for module test
!
if (allocated(test_fin)) deallocate(test_fin)
allocate(test_fin(Nwl,NSP))
if (allocated(test_fout)) deallocate(test_fout)
allocate(test_fout(Nwl,NSP))

if (allocated(test_fain)) deallocate(test_fain)
allocate(test_fain(nmu,NSP))
if (allocated(test_faout)) deallocate(test_faout)
allocate(test_faout(nmu,NSP))
!
! For module camera
!
if (allocated(cam_ft)) deallocate(cam_ft)
allocate(cam_ft(Nwl,cam_ndir))
if (allocated(cam_fr)) deallocate(cam_fr)
allocate(cam_fr(Nwl,cam_ndir))
if (allocated(cam_fs)) deallocate(cam_fs)
allocate(cam_fs(Nwl,cam_ndir))
if (allocated(cam_fd)) deallocate(cam_fd)
allocate(cam_fd(Nwl,cam_ndir))
if (allocated(cam_vn)) deallocate(cam_vn)
allocate(cam_vn(3,cam_ndir))
if (allocated(cam_sz)) deallocate(cam_sz)
allocate(cam_sz(cam_ndir))

call computation(Np, Ls, Lat, LTST, TauD, TauC, CloudH, Albedo, SType, &
                 NSP, NSPMax, SPZ, SPA, Nwl, NwlMax, wl, ss, &
                 ft, fs, fd, fr, nmu, fat, fas, fad, far)

! write outputs
! Open file to write
open(unit = 20, action="write", file=sa_spectral)

! Write outputs
write(20,*)"lambda1[nm],lambda2,(lambda1+lambda2)/2,Ft[mW/cm2/5nm],Fs,Fd,Fr"
dw2=0.5*(wl(2)-wl(1))
do i = 1, Nwl
  write(20,fmt='(f7.1,f7.1,f7.1,400(1x,e9.3))')  wl(i)-dw2, wl(i)+dw2, wl(i), &
            ft(i,:), fs(i,:), fd(i,:), fr(i,:)
end do
! Close file
close(20)

! write outputs for camera
! Open file to write
open(unit = 20, action="write", file=cam_spectral)

! Write outputs
write(20,*)"lambda1[nm],lambda2,(lambda1+lambda2)/2,Ft[mW/cm2/5nm],Fs,Fd,Fr"
dw2=0.5*(wl(2)-wl(1))
do i = 1, Nwl
  write(20,fmt='(f7.1,f7.1,f7.1,400(1x,e9.3))')  wl(i)-dw2, wl(i)+dw2, wl(i), &
            cam_ft(i,:), cam_fs(i,:), cam_fd(i,:), cam_fr(i,:)
end do
! Close file
close(20)

! clean up
if (allocated(ft)) deallocate(ft)
if (allocated(fs)) deallocate(fs)
if (allocated(fd)) deallocate(fd)
if (allocated(fr)) deallocate(fr)

! write outputs for the top of the atmosphere
! Open file to write
open(unit = 20, action="write", file=top_spectral)

! Write outputs
write(20,*)"lambda1[nm],lambda2,(lambda1+lambda2)/2,Fin[mW/cm2/5nm],Fout"
dw2=0.5*(wl(2)-wl(1))
do i = 1, Nwl
  write(20,fmt='(f7.1,f7.1,f7.1,400(1x,e9.3))')  wl(i)-dw2, wl(i)+dw2, wl(i), &
            top_fin(i), top_fout(i)
end do
! Close file
close(20)

if (allocated(top_fin)) deallocate(top_fin)
if (allocated(top_fout)) deallocate(top_fout)

! write outputs
! Open file to write
open(unit = 20, action="write", file=sa_angular)

! Write outputs
write(20,*)"zen.ang1[deg], zen.ang.2, zen.ang, Ft, Fs, Fd, Fr"
do i = 1, nmu
        
    write(20,fmt='(f5.1,f5.1,f5.1,400(1x,e9.3))') &
    (i-1)*90/real(nmu), i*90/real(nmu), (real(i)-.5)*90/real(nmu), &
    fat(i,:), fas(i,:), fad(i,:), far(i,:)
end do

! Close file
close(20)

! clean up
if (allocated(fat)) deallocate(fat)
if (allocated(fas)) deallocate(fas)
if (allocated(fad)) deallocate(fad)
if (allocated(far)) deallocate(far)

end program main

! ***************************************************************

   !!!--- PART 3 ---!!!

!-- Subroutines used by program "main" --! 

subroutine readinput(input,ssname,Np, Ls, Lat, LTST, NSP, NSPMax, SPZ, SPA, Albedo,&
                     SType, TauD, TauC, CloudH)
use constants
use camera
use boundary
implicit none

character(30),intent(in) :: input
character(30),intent(out) :: ssname
integer, intent(out) :: Np
real(8), intent(out) :: Ls
real(8), intent(out) :: Lat
real(8), intent(out) :: LTST
integer, intent(out) :: NSP
integer, intent(in) :: NSPMax 
real(8), dimension(NSPMax), intent(out) :: SPZ
real(8), dimension(NSPMax), intent(out) :: SPA
real(8), intent(out) :: Albedo
integer, intent(out) :: SType
real(8), intent(out) :: TauD
real(8), intent(out) :: TauC
real(8), intent(out) :: CloudH
! private
integer :: i ! loop counting
integer :: e ! reading error status

! Open file to read
open(unit = 10, action="read", file=input)

read  (10, *,iostat=e) sa_spectral
if (e .ne. 0) then
  write(*,*)"Problem reading filename for the spectral output on SA"
  write(*,*)"Using default name  = spectral"
  sa_spectral = 'spectral'
end if

read  (10, *,iostat=e) sa_angular
if (e .ne. 0) then
  write(*,*)"Problem reading filename for the angular output on SA"
  write(*,*)"Using default name  = angular"
  sa_angular = 'angular'
end if

read  (10, *,iostat=e) cam_spectral
if (e .ne. 0) then
  write(*,*)"Problem reading filename for the spectral output of camera"
  write(*,*)"Using default name  = cam_spectral"
  cam_spectral = 'cam_spectral'
end if

read  (10, *,iostat=e) top_spectral
if (e .ne. 0) then
  write(*,*)"Problem reading filename for the spectral output on the top of the atmosphere"
  write(*,*)"Using default name  = top_spectral"
  top_spectral = 'top_spectral'
end if

read  (10, *,iostat=e) ssname
if (e .ne. 0) then
  write(*,*)"Problem reading filename with the solar spectrum"
  write(*,*)"Using default name  = ss.csv"
  top_spectral = 'ss.csv'
end if

! Read inputs
read(10,*,iostat=e)Np
if (e .ne. 0) then
  write(*,*)"Problem reading integer number of photon per band Np"
  write(*,*)"Using default value Np = 1000000"
  Np = 1000000
end if
read(10,*,iostat=e)Ls
if (e .ne. 0) then
  write(*,*)"Problem reading real number solar longitude Ls"
  write(*,*)"Using default value Ls = 0.0"
  Ls = 0.0
end if
read(10,*,iostat=e)Lat
if (e .ne. 0) then
  write(*,*)"Problem reading real number observation latitude Lat"
  write(*,*)"Using default value Lat = 0.0"
  Lat = 0.0
end if
read(10,*,iostat=e)LTST
if (e .ne. 0) then
  write(*,*)"Problem reading real number Local True Solar Time LTST"
  write(*,*)"Using default value LTST = 12.0 (hour)"
  LTST = 12.0
end if
read(10,*,iostat=e)Albedo
if (e .ne. 0) then
  write(*,*)"Problem reading real number surface albedo Albedo"
  write(*,*)"Using default value Albedo = 0.2"
  Albedo = 0.2
end if
read(10,*,iostat=e)SType
if (e .ne. 0) then
  write(*,*)"Problem reading integer number surface type SType"
  write(*,*)"Using default value SType = 1"
  SType = 1
end if
read(10,*,iostat=e)TauD
if (e .ne. 0) then
  write(*,*)"Problem reading real number dust optical depth TauD"
  write(*,*)"Using default value TauD = 0.5"
  TauD = 0.5
end if
read(10,*,iostat=e)TauC
if (e .ne. 0) then
  write(*,*)"Problem reading real number cloud optical depth TauC"
  write(*,*)"Using default value TauC = 0.0"
  TauC = 0.2
end if
read(10,*,iostat=e)CloudH
if (e .ne. 0) then
  write(*,*)"Problem reading real number cloud height CloudH"
  write(*,*)"Using default value CloudH = 0.7*TauD"
  CloudH = 0.7 * TauD
end if
read(10,*,iostat=e)NSP
if (e .ne. 0) then
  write(*,*)"Problem reading integer number"
  write(*,*)"number of direction of solar panel NSP"
  write(*,*)"Using default value NSP = 1"
  NSP = 1
end if
do i = 1, NSP
 read(10,*,iostat=e)SPZ(i)
 if (e .ne. 0) then
   write(*,*)"Problem reading real number"
   write(*,*)"zenith angle of normal vector to solar panel SPZ"
   write(*,*)"at direction ",i
   write(*,*)"Using default value SPZ = 0.0"
   SPZ(i) = 0.0
 end if
 read(10,*,iostat=e)SPA(i)
 if (e .ne. 0) then
   write(*,*)"Problem reading real number"
   write(*,*)"azimuth angle of normal vector to solar panel SPA"
   write(*,*)"at direction ",i
   write(*,*)"Using default value SPA = 0.0"
   SPA(i) = 0.0
 end if
end do

read(10,*,iostat=e)cam_opening
if (e .ne. 0) then
  write(*,*)"Problem reading real number camera half-opening angle"
  write(*,*)"Using default value = 180.0"
  cam_opening = 180.0
end if
read(10,*,iostat=e)cam_ndir
if (e .ne. 0) then
  write(*,*)"Problem reading integer number"
  write(*,*)"number of direction of camera"
  write(*,*)"Using default value cam_ndir = 1"
  cam_ndir = 1
end if

if (allocated(cam_dira)) deallocate(cam_dira)
allocate(cam_dira(cam_ndir))
if (allocated(cam_dirz)) deallocate(cam_dirz)
allocate(cam_dirz(cam_ndir))
if (allocated(cam_a)) deallocate(cam_a)
allocate(cam_a(cam_ndir))

do i = 1, cam_ndir
 read(10,*,iostat=e)cam_dirz(i)
 if (e .ne. 0) then
   write(*,*)"Problem reading real number"
   write(*,*)"zenith angle of camera axis vector"
   write(*,*)"at direction ",i
   write(*,*)"Using default value = 0.0"
   cam_dirz(i) = 0.0
 end if
 read(10,*,iostat=e)cam_dira(i)
 if (e .ne. 0) then
   write(*,*)"Problem reading real number"
   write(*,*)"azimuth angle of camera axis vector"
   write(*,*)"at direction ",i
   write(*,*)"Using default value = 0.0"
   cam_dira(i) = 0.0
 end if
end do

! Close file
close(10)

! Check the validity of inputs
!! check Np
if (Np .le. 0) then
  write(*,*)"Input number of photon per band is"
  write(*,*)"not positive."
  write(*,*)"Using default value Np= 1000000"
  Np = 1000000
end if
!! check Ls

!! check Lat
if ((Lat .lt. -90.0) .or. (Lat .gt. 90.0)) then
  write(*,*)"Input solar panel's latitude is"
  write(*,*)"not a real number between -90 and 90."
  write(*,*)"Using default value Lat= 0.0"
  Lat = 0.0
end if
!! check LTST

!! check TauD
if (TauD .lt. 0.0) then
  write(*,*)"Input dust optical depth at reference wavelength is"
  write(*,*)"not a real number greater than 0."
  write(*,*)"Using default value TauD = 0.5"
  TauD = 0.5
end if
!! check TauC
if (TauC .lt. 0.0) then
  write(*,*)"Input cloud optical depth at reference wavelength is"
  write(*,*)"not a real number greater than or equal to 0."
  write(*,*)"Using default value TauC = 0.0"
  TauC = 0.0
end if
!! check CloudH
if ((CloudH .lt. 0.0) .or. (CloudH .ge. TauD) ) then
  write(*,*)"Input cloud height in term of optical thickness from surface"
  write(*,*)"at reference wavelength is"
  write(*,*)"not a real number greater than 0 and less than dust optical depth."
  write(*,*)"Using default value CloudH = 0.7*TauD"
  CloudH = 0.7*TauD
end if
!! check Albedo
if (Albedo .lt. 0.0) then
  write(*,*)"Input spectrally integrated surface albedo"
  write(*,*)"is not a real number greater than or equal to 0."
  write(*,*)"Using default value Albedo = 0.2"
  Albedo = 0.2
end if
!! check SType
if ((SType .lt. 0) .or. (SType .gt. 3)) then
  write(*,*)"Input surface spectral profile type"

  write(*,*)"Using default value Stype = 1"
  SType = 1
end if

!! check SPZ
do i = 1, NSP
 if ((SPZ(i) .lt. 0.0) .or. (SPZ(i) .gt. 90.0)) then
   write(*,*)"Input zenith angle of normal vector to solar panel is"

   write(*,*)"at direction ",i
   write(*,*)"Using default value SPZ = 0.0"
   SPZ(i) = 0.0
 end if
end do
!! check SPA

write(*,*)"Inputs to computations:"
write(*,*)" Number of photon per band, Np = ", Np
write(*,*)" Solar Longitude, Ls = ", Ls, "degree"
write(*,*)" Local Latitude, Lat = ", Lat, "degree"
write(*,*)" Local True Solar Time, LTST = ", LTST, "hour"
write(*,*)" Surface spectrally integrated albedo, Albedo = ", Albedo
write(*,*)" Surface Type, SType = ", SType
write(*,*)" Dust optical depth, TauD = ", TauD
write(*,*)" Cloud optical depth, TauC = ", TauC
write(*,*)" Cloud height, CloudH = ", CloudH
write(*,*)" Number of solar panel, NSP = ",NSP 
do i = 1, NSP
 write(*,*)" Orientation of solar panel number",i
 write(*,*)"  Solar panel zenith angle, SPZ = ", SPZ(i), "degree"
 write(*,*)"  Solar panel azimuth angle, SPA = ", SPA(i), "degree"
end do

! Convert from degree to radian
Lat = Lat/180.0*Pi
Ls = Ls/180.0*Pi
SPZ = SPZ/180.0*Pi
SPA = SPA/180.0*Pi
!!!!!!!!!!
!! checkcam_dirz
do i = 1, cam_ndir
 if ((cam_dirz(i).lt. 0.0) .or. (cam_dirz(i).gt. 90.0)) then
   write(*,*)"Input camera zenith angle is"

   write(*,*)"at direction ",i
   write(*,*)"Using default value = 0.0"
   cam_dirz(i) = 0.0
 end if
end do

write(*,*)" Camera half-opening angle =", cam_opening, "degree"
write(*,*)" Number of camera directions = ",cam_ndir
do i = 1, cam_ndir
 write(*,*)" Orientation of camera number",i
 write(*,*)"  Camera zenith angle  = ", cam_dirz(i), "degree"
 write(*,*)"  Camera azimuth angle = ", cam_dira(i), "degree"
end do

cam_dirz = cam_dirz/180.0*Pi
cam_dira = cam_dira/180.0*Pi
cam_opening = cam_opening/180.0*Pi
cam_copening = cos(cam_opening)
!!!!!!!!!!
end subroutine readinput

! ***************************************************************

subroutine readss(ssname, Nwl, NwlMax, wl, ss)
implicit none
character(30),intent(in) :: ssname
integer, intent(out) :: Nwl
integer, intent(in) :: NwlMax
real(8), dimension(NwlMax), intent(inout) :: wl
real(8), dimension(NwlMax), intent(inout) :: ss
! private
character(len=60) :: a ! dummy reading character to skip the first line of "ss.csv"
real(8) :: x,y ! dummy reading real number in reading "ss.csv"
integer :: i ! loop counting
integer :: e ! reading error status

! read "ss.csv" (solar spectrum file, in CSV format)
!!! get Nwl
write(*,*)"Reading solar spectrum..."
open(unit=10, action="read", file=ssname)
read(10,*)a
i = 0
reading: do
 read(10,*,iostat=e)x,y
 if (e .eq. 0) then
  i = i + 1
 else
  exit reading
 end if
end do reading
close(10)
Nwl = i
if (Nwl .lt. 2) then
 write(*,*)"Error reading solar spectrum."
 write(*,*)"Number of band is less than 2."
 write(*,*)"Calculation stop."

end if
!!! get wl, ss
open(unit=10, action="read", file=ssname)
read(10,*)a
do i = 1, Nwl
 read(10,*)wl(i),ss(i)
end do
close(10)
write(*,*)"Done reading solar spectrum, number of band: ",Nwl
end subroutine readss

! ***************************************************************

subroutine Computation(Np,Ls, Lat, LTST, TauD, TauC, CloudH, Albedo, SType, &
                 NSP, NSPMax, SPZ, SPA, Nwl, NwlMax, wl, ss, &
                 ft, fs, fd, fr, nmu, fat, fas, fad, far)
use constants ! Define Pi, 2*Pi, and initiate seed for random generator
use camera
use boundary
implicit none
integer, dimension(NwlMax), intent(in) :: Np ! Number of photon per band
real(8), intent(in) :: Ls ! Solar longitude (radian)
real(8), intent(in) :: Lat ! Latitude of solar panel on the planet (radian)
real(8), intent(in) :: LTST ! Local True Solar Time
real(8), intent(in) :: TauD ! Optical depth of dust at reference wavelength 
real(8), intent(in) :: TauC ! Optical depth of cloud at reference wavelength 
real(8), intent(in) :: CloudH ! Height of cloud layer
                           ! in term of optical thickness from surface
real(8), intent(in) :: Albedo ! Spectral integrated surface albedo
integer, intent(in) :: SType ! Surface spectral profile type
integer, intent(in) :: NSP ! Number of solar panel direction
integer, intent(in) :: NSPMax ! Over dimensioning
real(8), dimension(NSPMax), intent(in) :: SPZ ! Zenith angle of normal vector to solar panel
!                          compared to local zenith (radian)
!                          SPZ = 0 -> normal vector is vertical
!                          (solar panel horizontal)
real(8), dimension(NSPMax), intent(in) :: SPA ! Azimuth angle of normal vector to solar panel
!                          compared to local North (radian)
!                          SPA = 0 -> North
!                          SPA = Pi/2 -> East
!                          SPA = Pi -> South
!                          SPA = 3Pi/2 -> West
integer, intent(in) :: Nwl ! number of wavelength
integer, intent(in) :: NwlMax ! Over dimensioning
real(8), intent(in), dimension(NwlMax) :: wl ! list of wavelength
real(8), intent(in), dimension(NwlMax) :: ss ! solar spectrum power
real(8), intent(out), dimension(Nwl,NSP) :: ft ! total solar panel's flux
real(8), intent(out), dimension(Nwl,NSP) :: fs ! received scattered flux
real(8), intent(out), dimension(Nwl,NSP) :: fd ! received direct flux
real(8), intent(out), dimension(Nwl,NSP) :: fr ! received reflected flux
integer, intent(in) :: nmu ! number of zenith agle in angular flux
! spectrally integrated angular flux
real(8), dimension(nmu,NSP), intent(out) :: fat, fas, fad, far 

! Private variables
integer :: i ! loop counting
real(8) :: MinI ! intensity below which photon disappear 
integer, parameter :: nl=3 ! number of vertical atmospheric layers 
real(8), dimension(nl) :: stl ! scattering optical height at upper
real(8), dimension(nl) :: atl ! absorption optical height at upper
real(8), dimension(nl) :: ol ! at each layer, wl(i) = (1-ssa)/ssa,
                          ! where ssa is single scattering albedo
real(8), dimension(nl) :: gl ! assymmetry factor of each layer
real(8), dimension(NSP) :: a ! azimuth angle of normal vector to solar pannel,
!           with respect to solar direction (radian)
!           a = 0 -> normal vector is in solar direction
real(8) :: csza ! cosine of solar zenith angle (radian)
real(8) :: dec ! Sun declination viewed from solar panel (radian)
real(8) :: sdec ! sine of dec
real(8) :: cdec ! cosine of dec
real(8) :: RA ! Sun Right Ascension viewed from solar panel (radian)
real(8) :: cRA ! cosine of RA
real(8) :: sRA ! sine of RA
real(8) :: d ! distance to Sun (AU)
real(8) :: sLat ! sine of Lat
real(8) :: cLat ! cosine of Lat
  
! Private data
real(8) :: p ! Semi-latus rectum of Martian elliptic orbit (AU)
real(8) :: e ! Eccentricity of Martian elliptic orbit 
real(8) :: t ! Angle from Ls=0 to the perihelion (radian)
real(8) :: so ! sin(Obliquity of Martian axis)
! Martian dust optical properties
real(8), dimension(Nwl) :: dw ! single scattering albedo
real(8), dimension(Nwl) :: dg ! asymmetry factor
real(8), dimension(Nwl) :: dt ! ratio Qext to reference wavelength  
! Martian water ice optical properties
real(8), dimension(Nwl) :: iw ! single scattering albedo
real(8), dimension(Nwl) :: ig ! asymmetry factor
real(8), dimension(Nwl) :: it ! ratio Qext to reference wavelength
real(8), dimension(Nwl) :: awl ! surface albedo at given wavelength band
real(8), dimension(Nwl) :: ss_awl ! ss * awl
integer :: k ! interpolation index
integer, parameter :: nicevis=7, ndatavis=53 ! size of optical properties data
real(8) :: longicevis(nicevis) ! wavelength in dust data
real(8) :: omegicevis(nicevis) ! dust single scattering albedo
real(8) :: gicevis(nicevis) ! dust asymmetry factor
real(8) :: epicevis(nicevis) ! dust Qext
real(8) :: longdatavis(ndatavis) ! wavelenth in water ice data
real(8) :: omegdatavis(ndatavis) ! water ice single scattering albedo
real(8) :: gdatavis(ndatavis) ! water ice asymmetry factor
real(8) :: epdatavis(ndatavis) ! water ice Qext
real(8) :: ref ! temporary referencing value of Qext
integer, parameter :: nalbedo=24 ! size of albedo spectral profile data
real(8) :: longalbedo(nalbedo) ! wavelength in albedo spectral data
real(8) :: albedoprofile(0:3, nalbedo) ! surface albedo spectral profile
real(8) :: iss ! integrated solar spectrum power
!$ integer,external :: OMP_GET_THREAD_NUM
!$ integer,external :: OMP_GET_NUM_THREADS
integer :: nthreads,id
! Martian dust optical properties
! taken from Ockert-Bell et al, 
! Journal of Geophysical Research, Volume 102, Issue E4, p. 9039-905, 1997
! excepting extrapolation at 0.2 micron
data longdatavis/&
       200.000,&       
       263.000,  325.429,  387.857,  450.286,  512.714,&            
       575.143,  637.571,  700.000,  800.000, 1173.330,&            
      1546.670, 1920.000, 2130.000, 2180.610, 2231.210,&            
      2281.820, 2332.420, 2383.030, 2433.640, 2484.240,&            
      2534.850, 2585.450, 2636.060, 2686.670, 2737.270,&            
      2787.880, 2838.480, 2889.090, 2939.700, 2990.300,&            
      3040.910, 3091.520, 3142.120, 3192.730, 3243.330,&            
      3293.940, 3344.550, 3395.150, 3445.760, 3496.360,&            
      3546.970, 3597.580, 3648.180, 3698.790, 3749.390,&            
      3800.000, 3900.000, 4118.920, 4337.840, 4556.760,&            
      4775.680, 4994.600/ ! in nanometer                            
data epdatavis/&
      2.14339,&
      2.14339, 2.16595, 2.18708, 2.20890, 2.23175,&                 
      2.24548, 2.26126, 2.27651, 2.30520, 2.43673,&                 
      2.34511, 2.52108, 2.79911, 2.87472, 2.95200,&                 
      3.02957, 3.10616, 3.18081, 3.25293, 3.32158,&                 
      3.38675, 3.44873, 3.50445, 3.54464, 3.49754,&                 
      3.42569, 3.39450, 3.38017, 3.37082, 3.41011,&                 
      3.44994, 3.48812, 3.52923, 3.57007, 3.60009,&                 
      3.62619, 3.64563, 3.66110, 3.66438, 3.66035,&                 
      3.65207, 3.64031, 3.62001, 3.59644, 3.56870,&                 
      3.53991, 3.47203, 3.30704, 3.12001, 2.91671,&                 
      2.70735, 2.50083/  
data omegdatavis/&
      0.573682,&
      0.573682, 0.600724, 0.656430, 0.725456, 0.803155,&            
      0.890601, 0.948475, 0.962515, 0.961231, 0.949736,&            
      0.946121, 0.944339, 0.949037, 0.951986, 0.954277,&            
      0.956162, 0.957232, 0.957921, 0.958241, 0.958360,&            
      0.957918, 0.956473, 0.954736, 0.940937, 0.879367,&            
      0.823176, 0.797833, 0.783503, 0.772287, 0.788666,&            
      0.805719, 0.822459, 0.841554, 0.861491, 0.877158,&            
      0.892118, 0.905423, 0.918459, 0.925683, 0.932010,&            
      0.937806, 0.943465, 0.946053, 0.948416, 0.949450,&            
      0.950469, 0.951126, 0.950459, 0.950340, 0.947833,&            
      0.942340, 0.933691/                                           
data gdatavis/&
      0.933070,&
      0.933070, 0.918237, 0.892640, 0.863802, 0.832585,&            
      0.800394, 0.777051, 0.768704, 0.762508, 0.738694,&            
      0.659360, 0.639037, 0.664496, 0.671491, 0.678935,&            
      0.686440, 0.694295, 0.701983, 0.709680, 0.717115,&            
      0.724872, 0.733454, 0.741884, 0.756301, 0.780524,&            
      0.795635, 0.799233, 0.799490, 0.798015, 0.790548,&            
      0.782839, 0.777044, 0.772010, 0.766955, 0.763942,&            
      0.761222, 0.759424, 0.757674, 0.757574, 0.757445,&            
      0.757289, 0.757035, 0.757403, 0.757684, 0.758007,&            
      0.758309, 0.758485, 0.757120, 0.754178, 0.750413,&            
      0.745882, 0.740723/                                           
! Water ice optical properties
! Taken from Warren, 1984
data longicevis/ &
     103.3000,400.0000,800.0000,1000.000,1504.000, &
     2000.000,2600.000/ ! in nanometer
data omegicevis/ &
     0.5478598,0.9999998,0.9999930,0.9999391,0.9852892, &
     0.9747255,0.9896471/
data gicevis/ &
     0.9487292,0.8501517,0.8084917,0.8064129,0.7796354, &
     0.8265753,0.9093526/
data epicevis/ &
     2.0594757,2.0556829,2.2896707,2.3257456,2.5099661, &
     2.9161227,2.8020720/
data longalbedo / & ! in micron
     0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 0.6, &
     0.65, 0.7, 0.75, 0.8, 0.85, 0.9, 0.95, 1.0, 1.05, &
     1.1, 1.15, 1.2, 1.25, 1.3, 1.9 /
data albedoprofile(0,:) / & ! spectrally flat surface
     1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1., &
     1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1./
data albedoprofile(1,:) / & ! Ockert-Bell et al, 1996
     0.05,0.04,0.03,0.03,0.03,0.05,0.08,0.12,0.16,0.23,0.27, &
     0.32,0.33,0.32,0.32,0.33,0.333,0.335,0.34,0.342,0.345,0.348,0.35,0.35/
data albedoprofile(2,:) / & ! Bellucci et al, 2006, bright Olympus Mons 
     0.07,0.065,0.06,0.06,0.07,0.07,0.1,0.15,0.24,0.305,0.34, &
     0.36,0.36,0.36,0.36,0.36,0.36,0.36,0.36,0.36,0.36,0.36,0.36,0.36 /
data albedoprofile(3,:) / & ! Bellucci et al, 2006, dark Oxia Palus
     0.07,0.065,0.06,0.06,0.07,0.07,0.1,0.115,0.16,0.195,0.2,0.205,0.195, &
     0.185,0.18,0.18,0.18,0.18,0.18,0.18,0.18,0.18,0.18,0.18 /

!-- Preparations--!

! Dust & Ice properties
!! data dust
! -- interpolation from internal data -- !
do i = 1, Nwl
 call Z2K(wl(i),ndatavis,longdatavis,k)
 dw(i) = ((wl(i)-longdatavis(k-1))*omegdatavis(k) &
            + (longdatavis(k)-wl(i))*omegdatavis(k-1))/&
              (longdatavis(k)-longdatavis(k-1))
 dg(i) = ((wl(i)-longdatavis(k-1))*gdatavis(k) &
            + (longdatavis(k)-wl(i))*gdatavis(k-1))/&
              (longdatavis(k)-longdatavis(k-1))
 dt(i) = ((wl(i)-longdatavis(k-1))*epdatavis(k) &
            + (longdatavis(k)-wl(i))*epdatavis(k-1))/&
              (longdatavis(k)-longdatavis(k-1))
end do
call Z2K(670.0_8,ndatavis,longdatavis,k) ! find index at 670nm
ref = ((670.0-longdatavis(k-1))*epdatavis(k) &
            + (longdatavis(k)-670)*epdatavis(k-1))/&
              (longdatavis(k)-longdatavis(k-1))  ! reference at 0.67 micron
dt = dt / ref
!! data water ice
! -- interpolation from internal data -- !
do i = 1, Nwl
 call Z2K(wl(i),nicevis,longicevis,k)
 iw(i) = ((wl(i)-longicevis(k-1))*omegicevis(k) &
            + (longicevis(k)-wl(i))*omegicevis(k-1))/&
              (longicevis(k)-longicevis(k-1)) 
 ig(i) = ((wl(i)-longicevis(k-1))*gicevis(k) &
            + (longicevis(k)-wl(i))*gicevis(k-1))/&
              (longicevis(k)-longicevis(k-1))
 it(i) = ((wl(i)-longicevis(k-1))*epicevis(k) &
            + (longicevis(k)-wl(i))*epicevis(k-1))/&
              (longicevis(k)-longicevis(k-1))
end do
call Z2K(670.0_8,nicevis,longicevis,k) ! find index at 670nm
ref = ((670.0-longicevis(k-1))*epicevis(k) &
            + (longicevis(k)-670)*epicevis(k-1))/&
              (longicevis(k)-longicevis(k-1))  ! reference at 0.67 micron
it = it / ref
! Surface property
! Interpolation of profile
longalbedo = longalbedo * 1000 ! convert to nanometer
do i = 1, Nwl
 call Z2K(wl(i),nalbedo,longalbedo,k)
 awl(i) = ((wl(i)-longalbedo(k-1))*albedoprofile(SType,k) &
            + (longalbedo(k)-wl(i))*albedoprofile(Stype,k-1))/&
              (longalbedo(k)-longalbedo(k-1))
 ss_awl(i) = ss(i) * awl(i)
end do
call Integrate(Nwl,wl(2)-wl(1),ss_awl,ref) ! Assuming wl is equally spaced
call Integrate(Nwl,wl(2)-wl(1),ss,iss) ! Assuming wl is equally spaced
awl = awl * Albedo * iss / ref

!!-- Geometry preparations --!!
! Assign p, e, t
p    = 1.510404 ! Semi-latus rectum of Martian elliptic orbit (AU)
e    = 9.3357898E-02 ! Eccentricity of Martian elliptic orbit 
t    = 1.908231      ! Angle from Ls=0 to the perihelion (radian)
so   = 0.4256214     ! sin(Obliquity of Martian axis) 
sLat = sin(Lat)
cLat = cos(Lat)

! Finding:
!   1. Sun Right Ascension, RA (radian)
! from:
!   2. Local True Solar Time, LTST (hour)
RA  = Pi*(1.0-LTST/12.0)
sRA = sin(RA)
cRA = cos(RA)

! Finding:
!   1. distance to Sun, d (AU)
!   2. Sun declination viewed from solar panel, dec (radian)
! from:
!   1. solar longitude, Ls (radian)
d = p/(1.0+e*cos(Ls+t)) ! From polar equation of ellipse
sdec = sin(Ls)*so ! To be checked
dec  = asin(sdec)  
cdec = cos(dec)

! Finding:
!   1. solar zenith angle, sza / Local horizontal (radian)
! from:
!   1. Local True Solar Time, LTST (hour)
!   2. solar declination viewed from solar panel, dec (radian)
!   3. latitude of solar panel, Lat (radian)
csza = sLat*sdec+cLat*cdec*cRA
! dot product of 2 vectors

!!!!!!!!!!
write(*,*)" distance to Sun = ",d,"AU" 
write(*,*)" solar zenith angle = ", dacos(csza)*180.0/pi, "degree"

!initialization of arrays
ft = 0.0
fr = 0.0
fs = 0.0
fd = 0.0
fat = 0.0
far = 0.0
fas = 0.0
fad = 0.0

cam_ft = 0.0
cam_fr = 0.0
cam_fs = 0.0
cam_fd = 0.0
 
top_fin  = 0.0
top_fout = 0.0

if (csza .le. 0.0) then
 write(*,*)"Sun is below the horizon" 
 return
endif 

! Finding
!   1a. azimuth angle of normal vector to solar panel
!       with respect to solar direction viewed from solar panel, a
!   1b. azimuth angle of camera axis 
!       with respect to solar direction viewed from camera, cam_a
! from:
!   1a. azimuth angle of normal vector to solar panel
!       with respect to East direction, SPA
!   1b. azimuth angle of camera axis 
!       with respect to East direction, cam_dira
!   2a. zenith angle of normal vector to solar panel, SPZ
!   2b. zenith angle of camera axis, cam_dirz
!   3.  latitude of solar panel, Lat
!   4.  solar declination, dec
!   5.  solar azimuth angle, sza
if (((cdec*sRA) .eq. 0.0) .and. ((sLat*cdec*cRA-cLat*sdec) .eq. 0.0)) then
 a     = SPA(1:NSP) ! some compilator need specfying value for atan2(0,0)  
 cam_a = cam_dira(1:cam_ndir) ! some compilator need specfying value for atan2(0,0)   
else
 a     = SPA(1:NSP) + atan2(cdec*sRA,sLat*cdec*cRA-cLat*sdec)
 cam_a = cam_dira(1:cam_ndir) + atan2(cdec*sRA,sLat*cdec*cRA-cLat*sdec) 
end if

!!-- Preparation for Monte Carlo --!!
MinI = 0.00000001
call Seeding(seed) ! Generate seed (shared in module Constants)
!-- Do Monte-Carlo simulations --!
write(*,*)"Computing..."
id = 0
nthreads = 1  
!OpenMP specific operations:   
!$ nthreads = OMP_GET_NUM_THREADS()
!$OMP PARALLEL &
!$OMP PRIVATE(i,id,ol,gl,stl,atl), &
!$OMP SHARED(fd,fs,fr,fad,fas,far,top_fin,top_fout,cam_fd,cam_fs,cam_fr)
!$ id = OMP_GET_THREAD_NUM()
!$ write(*,*) "thread=", id," of ",nthreads
!$OMP DO SCHEDULE(STATIC,1) REDUCTION(+:fad,fas,far)
do i = 1, Nwl
 ! prepare the atmospheric model
 ol(1) = (1-dw(i))/dw(i) 
 ol(2) = (1-iw(i))/iw(i)
 ol(3) = ol(1)
 gl(1) = dg(i) 
 gl(2) = ig(i)
 gl(3) = gl(1)
 stl(1) = CloudH*dw(i)*dt(i) 
 stl(2) = TauC*iw(i)*it(i)
 stl(3) = (TauD-CloudH)*dw(i)*dt(i) 
 atl = ol*stl
 atl(2) = atl(1)+atl(2)
 atl(3) = atl(2)+atl(3)
 stl(2) = stl(1)+stl(2)
 stl(3) = stl(2)+stl(3)
 ! do simulation
 write(*,*)" At wavelength number ",i," by ",id   

 call MonteCarlo(i,Nwl,Np(i),MinI,nl,stl,atl,ol,gl,awl(i),NSP,SPZ,a,-csza, &
         fd,fs,fr, nmu, fad, fas, far, ss(i)*csza/(d**2.0)) 
end do
!$OMP END DO
!$ write(*,*) id,"out"
!$OMP END PARALLEL
!End of OpenMP section
!-- Output --!
! multiplying with solar spectral power
! on the horizontal top atmosphere surface
! (mW falling on a cm2 of horizontal top atmosphere surface
!  divided by the band width 5nm) 
ft = fd + fs + fr
fat = fad + fas + far
do i = 1, NSP
 iss = sum(fat(:,i))
 if (iss .gt. 0.0) then 
   fat(:,i) = fat(:,i) / iss
   fad(:,i) = fad(:,i) / iss
   fas(:,i) = fas(:,i) / iss
   far(:,i) = far(:,i) / iss
 else if (iss .lt. 0.0) then
   write(*,*)"Error on calculation"
   write(*,*)"Found negative flux"
!   write(*,*)"Computation stop"
!   stop
 end if
end do

! Special "bonus"
write(*,*)"Spectrally integrated total flux (below ",wl(Nwl)/1000.0," microns):"
do i = 1, NSP
 write(*,*)" For solar panel direction ", i 
 write(*,*)"  Total flux received ",10*sum(ft(:,i))*(wl(2)-wl(1)),"W/m2"
end do

cam_ft = cam_fd + cam_fs + cam_fr
write(*,*)"CAMERA Spectrally integrated total flux (below ",wl(Nwl)/1000.0," microns):"
do i = 1, cam_ndir
 write(*,*)" For camera direction ", i 
 write(*,*)"  Total flux received ",10*sum(cam_ft(:,i))*(wl(2)-wl(1)),"W/m2"
end do
 
write(*,*)"TOP of the atmosphere: Spectrally integrated total flux (below ",wl(Nwl)/1000.0," microns):"
write(*,*)"  Total flux entering ",10*sum(top_fin(:))*(wl(2)-wl(1)),"W/m2"
write(*,*)"  Total flux outgoing ",10*sum(top_fout(:))*(wl(2)-wl(1)),"W/m2"
 
!  assuming ss.csv has data
!  * equally spaced wavelength
!  * solar spectrum has unit mW/(cm2 . unit of wavelength)  
end subroutine Computation

! ***************************************************************

   !!!--- PART 4 ---!!!

!-- Subroutines used by subroutine Computation --!

! Assuming unit flux on horizontal surface, any unit
! Result must be multiplied with the flux on the horizontal surface
subroutine MonteCarlo(iwl,Nwl,Np,MinI,nl,stl,atl,wl,gl,albedo,NSP,z,a,csza,fd,fs,fr, &
                       nmu, fad, fas, far, f0)
use constants ! use predefined Pi, 2*Pi and initialized random seed
use camera
use boundary
implicit none
integer, intent(in) :: iwl ! index of wavelength
integer, intent(in) :: Nwl ! number of wavelength
integer, intent(in) :: Np ! number of photon to launch
real(8), intent(in) :: MinI ! minimum intensity below which photon disappear 
integer, intent(in) :: nl ! number of atmospheric layer
real(8), intent(in), dimension(nl) :: stl ! scattering optical height at upper boundary of layer
! stl(1) = on surface
! stl(nl) = on top
real(8), intent(in), dimension(nl) :: atl ! absorption optical height at upper boundary of layer
real(8), intent(in), dimension(nl) :: wl ! at each layer, wl(i) = (1-ssa)/ssa,
                                      ! where ssa is single scattering albedo 
real(8), intent(in), dimension(nl) :: gl ! assymmetry factor of each layer
real(8), intent(in) :: albedo ! surface albedo
integer, intent(in) :: NSP ! number of solar panel direction
real(8), intent(in), dimension(NSP) :: z ! zenith angle of normal vector to solar pannel 
real(8), intent(in), dimension(NSP) :: a ! azimuth angle of normal vector to solar pannel,
!                       with respect to solar direction
!                       a = 0 -> normal vector is in solar direction
real(8), intent(in) :: csza ! cosine of solar zenith angle 
real(8), intent(out), dimension(Nwl,NSP) :: fd ! direct flux that fall on solar panel
real(8), intent(out), dimension(Nwl,NSP) :: fs ! scattered flux that fall on solar panel
real(8), intent(out), dimension(Nwl,NSP) :: fr ! received flux reflected from surface
integer, intent(in) :: nmu ! number zenithal bin in angular flux
real(8), dimension(nmu,NSP), intent(inout) :: far, fas, fad ! spectrally integrated angular flux
real(8), intent(in) :: f0 ! initial flux on horizontal surface of top atmosphere

! private variables
real(8) :: Intensity ! Intensity of photon
real(8) :: S  ! scattering free-path length 
real(8) :: PS1, PS2 ! scattering optical depth of photon
real(8) :: PA1, PA2 ! absorption optical depth of photon
real(8) :: D1 ! cos zenith angle of photon in Local Coordinate System
real(8) :: D2 ! azimuth angle of photon in Local Coordinate System
integer :: k ! index of layer in which photon is found
real(8) :: Mu ! Scattering angle in Local Coordinate System
real(8) :: x   ! Random real number between [0,1)
integer :: i ! Multiple Photon Loop counting
integer :: nscat ! integer counting scattering order
real(8), dimension(NSP) :: sz ! sine of z
real(8), dimension(3,NSP) :: vn ! normal unit vector to solar panel
real(8) :: sD1 ! sine of photon's zenith angle
integer :: ksp ! counting direction
real(8) :: random ! function random 
real(8), dimension(nmu,NSP) :: tfar, tfas, tfad ! temporary angular flux 
real(8) :: cz ! cosine of zenith angle between photon and normal
real(8) :: czd1 
integer :: kz ! index of zenithal bin  
real(8) :: nmuPi ! number of discrete bin in the whole Pi 

! Preparations
sz = sin(z)
vn(1,:) = sz*cos(a)
vn(2,:) = sz*sin(a)
vn(3,:) = cos(z)
cam_sz = sin(cam_dirz)
cam_vn(1,:) = cam_sz*cos(cam_a)
cam_vn(2,:) = cam_sz*sin(cam_a)
cam_vn(3,:) = cos(cam_dirz)

tfar = 0.0
tfas = 0.0
tfad = 0.0
nmuPi = nmu*2/pi
! Multiple scattering
MultiplePhotonLoop: do i = 1, Np ! Start loop of multiple-photon
   ! Initiation of Photon enter top atmosphere
   Intensity = 1.0
   PS1 = stl(nl) ! on top
   PA1 = atl(nl) ! on top
   D1 = csza
   D2 = 0.0
   nscat = 0
   top_fin(iwl) = top_fin(iwl)+Intensity*1.0
   
   OnePhotonLoop : do         ! Start loop of multiple-scattering
      x = random(seed)        ! Choose free Scattering Path
      if(x.gt.1.0)     x= 0.999999 ! to prevent S=0
      if(x.lt.arg_min) x= arg_min
      S = - log(x)            ! 4.3e-7 <= S <= log(arg_min)
 
      PS2 = PS1 + D1 * S      ! Move Photon hypothetically
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Path through top (i.e. D1>0 --> PS2>PS1)
      if (PS2 .ge. stl(nl)) then   ! hit top atmosphere
       ! absorption optical height       
       PA2 = atl(nl)          ! since this is exit from the top         
       x = (PA1-PA2)
       
       if(x.lt.0.0) then 
        if(abs(D1).le.1.0e-2*abs(x)) then
         Intensity = 0.0       ! since exp(-100)=3.7e-44
        else         
         Intensity = Intensity * exp(x/D1) ! absorption (here x<0 and D1>0)        
        endif
       endif
!      if(x.gt.0.0) x = 0.0 this correction of the rounding error (since PA2>=PA1 in the case of exit from the top)
!      if x==0 then Intensity is preserved
       top_fout(iwl) = top_fout(iwl)+Intensity*1.0 ! no need in correction due to incident angle

       exit OnePhotonLoop
      endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Path reach surface (i.e. D1<0 --> PS2<PS1)
      if(PS2 .le. 0.0) then ! reach surface
       ! Absorption
       if(PA1.gt.0.0) then
        if(abs(D1).lt.1.0e-2*PA1) then
         Intensity = 0.0     ! since exp(-100)=3.7e-44
        else 
         Intensity = Intensity * exp(PA1/D1)  ! absorption (here PA1>0 and D1<0)  
        endif
       endif 
       !-- Record photon --!
       x = 1.0-D1**2
       if(x.lt.0.0) x = 0.0
       sD1 = sqrt(x) ! sine of photon's zenith angle

       do ksp = 1, NSP
!         Tran :
        cz = dot_product( vn(:,ksp), (/sD1*cos(D2),sD1*sin(D2),D1/))
        if(cz.lt.-1.0) cz = -1.0
        if(cz.gt. 1.0) cz =  1.0
!         Francois
!         cz=cos(z(ksp))*D1+sz(ksp)*sD1*cos(a(ksp)) 
        if(cz.lt. 0) then
         if(abs(d1).lt.arg_min) then
          czd1 = cz/(-arg_min)  ! here cz<0 and -arg_min<0 --> czd1>0          
         else    
          czd1 = cz/d1          ! here cz<0 and d1<0 -->czd1>0
         endif

         kz = int(nmuPi * acos(-cz) + 1)
         if (nscat .eq. 0) then
          fd(iwl,ksp) = fd(iwl,ksp) + Intensity*czd1
          tfad(kz,ksp) = tfad(kz,ksp) + Intensity*czd1
         else
          fs(iwl,ksp) = fs(iwl,ksp) + Intensity*czd1
          tfas(kz,ksp) = tfas(kz,ksp) + Intensity*czd1
         end if
        end if ! if cz<0
       end do
! For camera
!
       do ksp = 1, cam_ndir
        cz = dot_product( cam_vn(:,ksp), (/sD1*cos(D2),sD1*sin(D2),D1/))
        if(cz.lt.-1.0) cz = -1.0
        if(cz.gt. 1.0) cz =  1.0

        if(cz .lt. 0 .and.abs(cz).gt.cam_copening) then
         if(abs(d1).lt.arg_min) then
          czd1 = cz/(-arg_min)  ! here cz<0 and -arg_min<0 --> czd1>0           
         else    
          czd1 = cz/d1          ! here cz<0 and d1<0 -->czd1>0
         endif
            
         if(nscat .eq. 0) then
          cam_fd(iwl,ksp) = cam_fd(iwl,ksp) + Intensity*czd1
         else
          cam_fs(iwl,ksp) = cam_fs(iwl,ksp) + Intensity*czd1
         end if
        end if ! if cz<0
       end do !ksp for camera
         
       ! Check if photon is weak
       if(Intensity < MinI) exit OnePhotonLoop ! weak photon
       ! Move Photon to surface
       PS1 = 0.0        
       PA1 = 0.0
       ! Lambertian reflection

       D1 = random(seed)
       D2 = random(seed) * Pi2
       Intensity = Intensity * albedo * 2.0 * D1
!! equation Intensity=Intensity*albedo/Pi*D1 is wrong for photon packages       

       if(nscat .eq. 0) nscat = 1
       !-- Record reflected photon --!
       x = 1.0-D1**2
       if(x.lt.0.0) x = 0.0
       sD1 = sqrt(x) ! sine of photon's zenith angle

       do ksp = 1, NSP
        cz = dot_product( vn(:,ksp), (/sD1*cos(D2),sD1*sin(D2),D1/))
        if(cz.lt.-1.0) cz = -1.0
        if(cz.gt. 1.0) cz =  1.0

        if(cz.lt. 0) then
         if(d1.lt.arg_min) then
          czd1 = -cz/arg_min   ! here cz<0 and arg_min>0 --> czd1>0          
         else    
          czd1 = -cz/d1        ! here cz<0 and d1>0 -->czd1>0
         endif

         fr(iwl,ksp) = fr(iwl,ksp) + Intensity*czd1
         kz = int(nmuPi * acos(-cz) + 1)
         tfar(kz,ksp) = tfar(kz,ksp) + Intensity*czd1
        end if ! if cz<0
       end do
! for Camera
       do ksp = 1, cam_ndir
        cz = dot_product( cam_vn(:,ksp), (/sD1*cos(D2),sD1*sin(D2),D1/))
        if(cz.lt.-1.0) cz = -1.0
        if(cz.gt. 1.0) cz =  1.0

        if(cz.lt. 0.and.abs(cz).gt.cam_copening) then
         if(d1.lt.arg_min) then
          czd1 = -cz/arg_min   ! here cz<0 and arg_min>0 --> czd1>0          
         else    
          czd1 = -cz/d1        ! here cz<0 and d1>0 -->czd1>0
         endif
         cam_fr(iwl,ksp) = cam_fr(iwl,ksp) + Intensity*czd1
        end if ! if cz<0
       end do ! ksp for camera
         
       cycle ! next iteration of OnePhotonLoop i.e. continue path simulation
      endif         
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Path through the middle of the atmosphere
! 0<PS2<stl(nl)

      ! find which layer photon is belonging to
      call Z2K(PS2,nl,stl,k)    ! stl(k-1) < PS2 <=stl(k)
                                ! if PS2 < stl(1) then k = 1 
      ! absorption 
      if(abs(D1).lt.arg_min) then          
       Intensity = Intensity * exp(-S*wl(k))    ! absorption
       if(Intensity < MinI) exit OnePhotonLoop  ! weak photon
      else
       !find absorption optical height
       PA2 = atl(k) - (stl(k)-PS2)*wl(k)        ! PS2<=stl(k) --> PA2<=atl(k)
            
       x = PA1-PA2
       if(D1.lt.0.0.and.x.lt.0.0) x = 0.0
       if(D1.gt.0.0.and.x.gt.0.0) x = 0.0
       x = x/D1
       Intensity = Intensity * exp(x)           ! absorption
       if(Intensity < MinI) exit OnePhotonLoop  ! weak photon
       PS1 = PS2                                ! move photon (vertically)
       PA1 = PA2
      end if

      ! Find new scattering direction
      x = random(seed)
      Mu = (1+gl(k)**2-((1-gl(k)**2)/(1-gl(k)+2*gl(k)*x))**2)/2.0/gl(k)
      ! Henyey-Greenstein
      if (Mu .gt. 1.0) then
       Mu = 1.0 ! Avoid numerical instability of "f90" compiler
      else if (Mu .lt. -1.0) then
       Mu = -1.0 ! Avoid numerical instability of "f90" compiler
      end if  
      x = random(seed) * Pi2
      call L2G(Mu,x, D1,D2, D1,D2)
      if (nscat .eq. 0) nscat = 1

   end do OnePhotonLoop
end do MultiplePhotonLoop

! Normalisation of flux
fs(iwl,:) = fs(iwl,:) * f0 / real(Np)
fd(iwl,:) = fd(iwl,:) * f0 / real(Np)
fr(iwl,:) = fr(iwl,:) * f0 / real(Np)

cam_fs(iwl,:) = cam_fs(iwl,:) * f0 / real(Np)
cam_fd(iwl,:) = cam_fd(iwl,:) * f0 / real(Np)
cam_fr(iwl,:) = cam_fr(iwl,:) * f0 / real(Np)

top_fin(iwl)  = top_fin(iwl)  * f0 / real(Np) 
top_fout(iwl) = top_fout(iwl) * f0 / real(Np)

tfas = tfas * f0 / real(Np)
tfad = tfad * f0 / real(Np)
tfar = tfar * f0 / real(Np)
fas = fas + tfas
fad = fad + tfad
far = far + tfar 

end subroutine MonteCarlo

! ***************************************************************

subroutine Seeding(seed)
    ! Purpose: initializing the random generator
    !          with "seed" as function of time stamp
    !          which will be used as seed for the random function below
    ! We have new seed for each run at different date and time
    implicit none
    integer :: date(8)
    integer, intent(out) :: seed
    call Date_And_Time(values=date)
! If _SEEDING_OFF_ is defined, we impose random number generation seed
#ifdef _SEEDING_OFF_    
    date(8) = 999 ! millisecond
    date(7) = 59  ! second
    date(6) = 59  ! minute
    date(5) = 23  ! hour
#endif    
    seed = date(8)*86400 + date(7)*1440 + date(6)*24 + date(5)
end subroutine Seeding

! ***************************************************************

subroutine Z2K(z,N,Zn,k)
    ! Purpose: find index of a number among intervals
    ! Given array Zn of size N, in accending order,
    ! and given number z, we will locate
    ! the position k so that: Zn(k-1) < z <= Zn(k)
    ! If z > Z(N) then k = N+1
    ! If z < Z(1) then k = 1
    implicit none
    real(8), intent(in) :: z
    integer, intent(in) :: N                 ! size of Zn
    real(8), dimension(N), intent(in) :: Zn ! assumed ascending order
    integer, intent(out) :: k
    integer :: lower, middle
    if (z .eq. Zn(N)) then       !Set the output, careful with the endpoints.
       k = N
    else
       lower = 0      !Initialize lower
       k = N + 1      !and upper limits.
       do
          if ((k - lower) .le. 1) exit !Repeat until condition is satisfied
          middle = (k + lower) / 2   !Compute a midpoint,
          if (z .ge. Zn(middle)) then
             lower = middle          !and replace either the lower limit
          else
             k = middle              !or the upper limit, as appropriate.
          end if
       end do
    end if
end subroutine Z2K

! ***************************************************************

subroutine Integrate(N,h,F,I) ! Equally spaced abcsissa
    ! Purpose: do numerical integration
    implicit none
    integer, intent(in) :: N ! Number of abcsissa
    real(8), intent(in) :: h ! spacing of abcsissa
    real(8), dimension(N), intent(in) :: F ! function F(x)
    real(8), intent(out) :: I ! the integral
    if (N .lt. 1) then
       stop 'Integrate receives N < 1'
    else if (N .eq. 1) then
       I = h * F(1)
    else if (N .eq. 2) then
       I = h * (F(1) + F(2)) / 2.0 ! Trapezoidal rule
    else if (N .eq. 3) then
       I = h * (F(1) + 4.0*F(2) + F(3)) / 3.0 ! Simpson's rule
    else if (N .eq. 4) then
       I=h*(3.0*(F(1)+F(4))+9.0*(F(2)+F(3)))/8.0
       ! Simpson's 3/8 rule
    else if (N .eq. 5) then
       I = h*(14.0*(F(1)+F(5)) + &
            64.0*(F(2)+F(4)) + 24.0*F(3))/45.0 ! Bode's rule
    else
       I = sum(F(4:N-3))
       I = h*((9.0*(F(1)+F(N)) + 28.0*(F(2)+F(N-1)) + &
           23.0*(F(3)+F(N-2)))/24.0 + I)
    end if
end subroutine Integrate

! ***************************************************************
                                  
!-- Subroutines used by MonteCarlo --!

subroutine L2G(Y1,Y2,L1,L2,X1,X2)
    ! Purpose: convert direction vector from Local Coordinate to Global one
    use constants ! use predefined 2*Pi
    implicit none
    real(8), intent(in) :: Y1,Y2,L1,L2 ! Y = Vector in LCS
                     ! L = The vector shows LCS in GCS
    real(8), intent(out) :: X1,X2 ! X = the same vector Y in GCS
    real(8) :: sY, sL, sX, cX,cY !, t2
       sY = sqrt(1.0-Y1**2)
       sL = sqrt(1.0-L1**2)
       sX = sin(Y2)*sY
       cY = cos(Y2)*sY
       cX = Y1*sL + L1*cY
       X1 = Y1*L1-sL*cY
       if ((sX .eq. 0.0) .and. (cX .eq. 0.0)) then
         X2 = L2
       else 
         X2 = mod(L2 + atan2(sX, cX), Pi2)
       end if 
end subroutine L2G

! ***************************************************************


function random(iy)

INTEGER, INTENT(inout) :: iy

!.......................................................................
!     urand is a uniform random number generator based on theory and
!     suggestions given in d.e. knuth (1969), vol 2.
!     the integer iy should be initialized to an arbitrary integer prior
!     to the first call to urand. the calling program should not alter
!     the value of iy between subsequent calls to urand. values of urand
!     will be returned in the interval (0,1).
!.......................................................................

INTEGER :: ia,ic,itwo=2,m2=0,m,mic
DOUBLE PRECISION :: halfm
REAL :: s
real(8) :: random
SAVE

IF (m2==0) THEN
!
!     if first entry, compute machine integer word length
!
      m=1
10    m2=m
      m=itwo*m2
      if (m.gt.m2) go to 10
      halfm=m2

!
!     compute multiplier and increment for linear congruential method
!
      ia=8*idint(halfm*datan(1.d0)/8.d0)+5
      ic=2*idint(halfm*(0.5d0-dsqrt(3.d0)/6.d0))+1
      mic=(m2-ic)+m2
!
!     s is the scale factor for converting to floating point
!
                                 s=0.5/halfm
!
!     compute next random number
!
ENDIF

iy=iy*ia

!     the following statement is for computers which do not allow
!     integer overflow on addition

IF (iy.gt.mic) iy=(iy-m2)-m2
!
iy=iy+ic

!     the following statement is for computers where the word length
!     for addition is greater than for multiplication

IF (iy/2.gt.m2) iy=(iy-m2)-m2

!     the following statement is for computers where integer overflow
!     affects the sign bit

IF (iy.lt.0) iy=(iy+m2)+m2

random=float(iy)*s

END FUNCTION random
