      program read_inputs_from_mcd

!     ----------------------------------------------------------------------
!     This program can be used by users of the Mars Climate Database 
!     to fill the LMD Ilumination tool input file. 
!
!     - USE:   
!     This model replaces: 1) Surface albedo, 2) Dust optical depth, 
!     3) Cloud optical depth, 4) Cloud height, in the user supplied input
!     file with inputs from the MCD Version 5.3 and a surface albedo map. 
!     The original input file is saved in a backup file ending by ".orig") 
!
!     - COMPILATION:
!     This program compiles like any program calling the "call_mcd"
!     subroutine, as explained in the MCD documentation. 
!  
!     Authors: F. Forget, E. Millour (12/2018) 
!     ----------------------------------------------------------------------

      implicit none

! Illumination tool input data to be re-estimated
! ------------------------------------------------
      real TauDust ! Surface Dust optical depth at reference wavelength (0.67 micron)
      real TauCloud !  Cloud optical depth at reference wavelength (0.67 micron)
      real CloudHeight !  Cloud height (given by dust optical thickness below cloud)
      real albedo    !  Spectrally integrated surface albedo


! CALL_MCD arguments
! ------------------
! inputs:
      integer zkey ! flag to choose the type of z coordinates
      real xz     ! value of the z coordinate
      real xlon   ! east longitude (degrees)
      real xlat   ! north latitude (degrees)
      integer hireskey ! high resolution flag (0: off, 1: on) 
      integer datekey  ! date flag (0: Earth date 1: Mars date)
      double precision xdate !  solar longitude Ls (with datekey=1)
      real ls   ! for user input of Ls (if using "martian time")
      real localtime ! local time at longitude xlon (only if datekey=1)
      character(len=100) :: dset=" " ! path to MCD datasets; unset here
                                    ! (ie: defaults to "MCD_DATA/")
      integer dust  ! dust and solar EUV scenario
      integer perturkey ! perturbation type
      real seedin  ! random generator seed and flag (if perturkey=1,2,3 or 4)
                   ! coefficient to multiply std. dev. by (if perturkey=5)
      real gwlength ! Gravity wave wavelength (needed if perturkey=3 or 4)
      integer extvarkeys(100) ! extra output variables (1: yes, 0: no)
! outputs:
      real pres  ! atmospheric pressure
      real ro    ! atmospheric density
      real temp  ! atmospheric temperature
      real u     ! zonal wind
      real v     ! meridional wind
      real meanvar(5)  ! unperturbed values of main meteorological variables
      real extvar(100) ! extra output variables
      real seedout     ! current value of random generator seed index
      integer ier ! call_mcd status (=0 if all went well)

!     Other variables
!     ---------------
      integer,parameter :: nlinemax=300 ! max number of lines in input file
      integer k,l,nline_input
      character(len=150), dimension(nlinemax) :: line
      character(len=150) :: linek
      character(len=100) :: inputfile,backupfile
      integer,parameter :: nlevels=30 ! number of levels used to estimate cloud height
      real ice_density, ice_density_max,p_icemax,ps,ptop,x


!     =========================================================
!     To be modified if needed :

      dset="MCD_DATA/"    ! path to the MCD datafile directory 

!     Choice of dust scenario :
!     dust=1-3: typical climatology; dust=8: clearer conditions ("cold scenario")
!     dust=7: dustier conditions ("warm scenario"); dust=4-6: extreme global dust storm
      dust=1
!     =========================================================

      ! name of illumination tool input file to process
      write(*,*)'Name of input file to modify?'
      read(*,'(a)')inputfile
      write(*,*)'EAST longitude in deg [-180:180]?'
      read(*,*)xlon
      write(*,*) 'longitude: ',xlon
      write(*,*)'Other coordinates are read in file: ', trim(inputfile)
      write(*,*)'Lets read the environmental conditions in the MCD'

  
!    Reading input file and creating backup input file
!    ------------------------------------------------- 
      open(10,file=trim(inputfile),iostat=ier,status="old")
      if (ier.ne.0) then
        write(*,*) "ERROR: input file ",trim(inputfile)," not found!"
        stop
      endif

      backupfile=trim(inputfile)//".orig"
      open(20,file=trim(backupfile))

      do k=1,nlinemax
         read(10,fmt='(a)',end=99) line(k)
         write(20,fmt='(a)') trim(line(k))
      end do
 99   continue
      nline_input=k-1
      close (10)
      close (20)

      read(line(7),*) xdate ! Ls solar longitude
      read(line(8),*) xlat
      read(line(9),*) localtime

!    Calling the Mars Climate Database to get improved input data
!    ------------------------------------------------------------ 
      datekey=1
      xz=1.
      hireskey=1
      perturkey=1
      seedin=0.
      gwlength=0.
      extvarkeys(:)=0
      extvarkeys(19)= 1    ! as for surface pressure 
      extvarkeys(36)=1     ! ask for Dust column visible optical depth above surface
      extvarkeys(43)= 1    ! ask for  Water ice column (kg/m2)
      extvarkeys(44)= 1    ! ask for  Water ice mixing ratio (mol/mol)

      zkey=4 !  vertical coordinates will be pressure level (in Pa )

!     loop to evaluate the height of the clouds between the ~1 Pa and the surface
!     the height is defined as where the ice density (kg_ice/m3) is maximum

      ptop =1. ! lowest pressure level evaluated, Pa (i.e. higher altitude)
      ps =610. ! first guess of surface pressure, Pa (not used in practice)
      p_icemax = 100 ! initial guess of cloud level (Pa)
      ice_density_max = 0. ! initial max ice density (kg/m3)
      do l=1,nlevels
         x=(float(l-1)/float(nlevels-1))
         xz= x*ps + (1.-x)*ptop ! pressure level in Pa

         call call_mcd(zkey,xz,xlon,xlat,hireskey,
     &              datekey,xdate,localtime,dset,dust,
     &              perturkey,seedin,gwlength,extvarkeys,
     &              pres,ro,temp,u,v,meanvar,extvar,seedout,ier)
         if (ier.eq.0) then
            ps=extvar(19)  ! surface pressure (Pa)
            ice_density =  (18./44.)*extvar(44)*ro   ! kg_ice/m3
            if (ice_density.gt.ice_density_max) then
               ice_density_max=ice_density
               p_icemax =  pres ! cloud level (Pa)
            end if
         else
           write(*,*)'CALL_MCD ERROR !!'
           write(*,*)'         returned error code: ', ier
         end if
      end do

!     Dust opacity
      TauDust = extvar(36)

!     Cloud opacity = Ice_column*(4/3)*Qext/(rho_ice*reff)
!     with Ice_column (kg/m2), assuming Qext=2, reff=5um,rho_ice=920(kg/m3 
      TauCloud = extvar(43)*(4./3.)*2./(920.*5.E-6)

!     Height of cloud (given by dust optical thickness below cloud)
      CloudHeight = (1.-p_icemax/ps)*TauDust !  We assume a well mixed dust...

!     Estimation of the local surface albedo
!     -------------------------------------- 
      call get_albedo(xlon,xlat,albedo)

!     modifying input file content of line #k
!     --------------------------------------- 
      k=10 ! surface albedo
      linek=trim(line(k))
      write(line(k),fmt='(f5.3,a)')
     &            albedo," "//linek(7:len_trim(linek))

      k=12 ! Dust optical depth
      linek=trim(line(k))
      write(line(k),fmt='(f6.3,a)')
     &            TauDust," "//linek(8:len_trim(linek))

      k=13 ! Cloud optical depth
      linek=trim(line(k))
      write(line(k),fmt='(f6.3,a)')
     &            TauCloud," "//linek(8:len_trim(linek))

      k=14 !  Cloud height (given by dust optical thickness below cloud)
      linek=trim(line(k))
      write(line(k),fmt='(f6.3,a)')
     &            CloudHeight," "//linek(8:len_trim(linek))

!     writing updated input file
!     -------------------------- 
      open(10,file=trim(inputfile))
      do k=1,nline_input
         write(10,fmt='(a)') trim(line(k))
      end do
      close(10)
      
      write(*,*) "read_inputs_from_mcd run ended successfully"
      
      contains
      
      subroutine get_albedo(xlon,xlat,albedo)
      use netcdf
      implicit none
      real,intent(in) :: xlon ! east longitude (degrees)
      real,intent(in) :: xlat ! north latitude (degrees)
      real,intent(out) :: albedo
      ! local variables
      character(len=20) :: filename="albedo.nc" ! albedo file
      integer :: lonlen
      integer :: latlen
      real,allocatable :: lon(:)
      real,allocatable :: lat(:)
      real,allocatable :: alb(:,:)
      integer :: ilon,ilat
      integer :: i
      ! NetCDF stuff
      integer :: status ! NetCDF return code
      integer :: ncid ! NetCDF file ID
      integer :: dimid ! NetCDF dimension ID
      integer :: varid ! NetCDF variable ID


      ! 1. load albedo dataset
      ! 1.1 Open file
      status=nf90_open(filename,NF90_NOWRITE,ncid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to open datafile ",trim(filename)
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      
      ! 1.2. Identify/load dimensions
      ! Longitude
      status=nf90_inq_dimid(ncid,"longitude",dimid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find longitude dimension"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      status=nf90_inquire_dimension(ncid,dimid,len=lonlen)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find longitude length"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      ! allocate lon()
      allocate(lon(lonlen),stat=status)
      if (status.ne.0) then
        write(*,*)"Failed to allocate longitude"
      endif
      !
      status=nf90_inq_varid(ncid,"longitude",varid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find longitude varid"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      ! read longitude
      status=nf90_get_var(ncid,varid,lon)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to load longitude"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      
      ! Latitude
      status=nf90_inq_dimid(ncid,"latitude",dimid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find latitude dimension"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      status=nf90_inquire_dimension(ncid,dimid,len=latlen)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find latitude length"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      ! allocate latitude()
      allocate(lat(latlen),stat=status)
      if (status.ne.0) then
        write(*,*)"Failed to allocate latitude"
      endif
      !
      status=nf90_inq_varid(ncid,"latitude",varid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find latitude varid"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      ! read latitude
      status=nf90_get_var(ncid,varid,lat)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to load latitude"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      
      ! 1.3. Identify/load variables
      ! allocate alb
      allocate(alb(lonlen,latlen),stat=status)
      if (status.ne.0) then
        write(*,*)"Failed to allocate alb"
      endif
      !
      status=nf90_inq_varid(ncid,"albedo",varid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to find albedo varid"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      ! read alb
      status=nf90_get_var(ncid,varid,alb)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to load alb"
        write(*,*)trim(nf90_strerror(status))
        stop
      endif

      ! 1.4. Close file
      status=nf90_close(ncid)
      if (status.ne.nf90_noerr) then
        write(*,*)"Failed to close file: ",trim(filename)
        write(*,*)trim(nf90_strerror(status))
        stop
      endif
      
      ! 2. Find nearest indexes of nearest lon and lat
      ilon=0
      ! Knowing that lon(:) are monotonously increasing we can do the following
      if (xlon.le.lon(1)) then
        ilon=1
      else if (xlon.ge.lon(lonlen)) then
        ilon=lonlen
      else
        do i=1,lonlen
        ! Knowing that lon(:) are monotonously increasing we can do the following
          if ((xlon.ge.lon(i)).and.(xlon.le.lon(i+1))) then
           ! find the nearest neighbour between lon(i) and lon(i+1)
           if ((xlon-lon(i)).le.(lon(i+1)-xlon)) then
            ilon=i
           else
            ilon=i+1
           endif
           exit
          endif
        enddo ! of do i=1,nblon
      endif
      !write(*,*) "ilon=",ilon," lon(ilon)=",lon(ilon)
      ilat=0
      ! Knowing that lat(:) are monotonously decreasing we can do the following
      if (xlat.ge.lat(1)) then
        ilat=1
      else if (xlat.le.lat(latlen)) then
        ilat=latlen
      else
       do i=1,latlen
       ! Knowing that lat(:) are monotonously decreasing we can do the following
        if ((xlat.le.lat(i)).and.(xlat.ge.lat(i+1))) then
          ! find the nearest neighbour between lat(i) and lat(i+1)
          if ((lat(i)-xlat).le.(xlat-lat(i+1))) then
           ilat=i
          else
           ilat=i+1
          endif
          exit
        endif
       enddo ! of do i=1,nblat
      endif
      !write(*,*) "ilat=",ilat," lat(ilat)=",lat(ilat)

      ! 3. Resulting albedo is thus
      albedo=alb(ilon,ilat)
      
      end subroutine get_albedo

      end

